<?php

namespace Drupal\commerce_worldline\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\Core\Form\FormStateInterface;
use Sips\Passphrase;
use Sips\PaymentRequest;
use Sips\PaymentResponse;
use Sips\ShaComposer\AllParametersShaComposer;
use Symfony\Component\HttpFoundation\Request;

/**
 * Defines the SIPS Payment for Offsite purchases.
 *
 * @CommercePaymentGateway(
 *   id = "sips_payment",
 *   label = "SIPS Payment (Offsite)",
 *   display_label = "SIPS Payment (Offsite)",
 *    forms = {
 *     "offsite-payment" = "Drupal\commerce_worldline\PluginForm\OffsiteRedirect\SIPSPaymentRedirectForm",
 *   },
 *   payment_method_types = {"sips"},
 *   modes = {"TEST", "SIMU", "PRODUCTION"}
 * )
 */
class SIPSPaymentGateway extends OffsitePaymentGatewayBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'sips_interface_version' => '',
      'sips_passphrase' => '',
      'sips_merchant_id' => '',
      'sips_key_version' => '',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['sips_interface_version'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Interface version'),
      '#default_value' => $this->configuration['sips_interface_version'],
      '#required' => TRUE,
    ];

    $form['sips_passphrase'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Passphrase'),
      '#default_value' => $this->configuration['sips_passphrase'],
      '#required' => TRUE,
    ];

    $form['sips_merchant_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Merchant ID'),
      '#default_value' => $this->configuration['sips_merchant_id'],
      '#required' => TRUE,
    ];

    $form['sips_key_version'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Key version'),
      '#default_value' => $this->configuration['sips_key_version'],
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);
      $this->configuration['sips_interface_version'] = $values['sips_interface_version'];
      $this->configuration['sips_passphrase'] = $values['sips_passphrase'];
      $this->configuration['sips_merchant_id'] = $values['sips_merchant_id'];
      $this->configuration['sips_key_version'] = $values['sips_key_version'];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function onReturn(OrderInterface $order, Request $request) {
    parent::onReturn($order, $request);

    /** @var \Drupal\commerce_payment\Entity\PaymentMethodInterface $payment_method */
    $payment_method = $order->payment_method->entity;

    /* @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\PaymentGatewayInterface $payment_gateway */
    $payment_gateway = $order->payment_gateway->entity;

    // Prepare the PaymentRequest from the global request data.
    $payment_response = new PaymentResponse($request->request->all());

    $transaction_reference = $payment_response->getParam('transactionReference');

    $payment = $this->entityTypeManager->getStorage('commerce_payment')->load($transaction_reference);

    // Gets the payment gateway configuration.
    $config = $payment_gateway->getPlugin()->getConfiguration();

    // Prepares the validation.
    $passphrase = new Passphrase($config['sips_passphrase']);
    $shaComposer = new AllParametersShaComposer($passphrase);

    if (!$payment_response->isValid($shaComposer)) {
      // This is not a valid response either because:
      // - the passphrase is not matching the request
      // - the transaction reference from the request is not matching the
      //   payment reference
      // - the remote state is not pending
      //
      // We're aborting the request here.
      \Drupal::logger('commerce_worldline')
        ->warning('User arrived to commerce_worldline.handle_response without valid information: ' .
          '[transaction reference equals: @transactionReference - @transactionReferencePayment]' .
          '[Is remote state pending: @remoteState] [Valid SIPS answer: @valid].', [
            '@transactionReference' => $transaction_reference,
            '@transactionReferencePayment' => $payment->getRemoteId(),
            '@remoteState' => $payment->getRemoteState(),
            '@valid' => $payment_response->isValid($shaComposer) ? 'Yes' : 'No',
          ]);

      drupal_set_message($this->t('An error occurred while processing your request.'), 'error');
      throw new PaymentGatewayException();
    }

    // Update the payment method with the response code.
    $code = $payment_response->getParam('RESPONSECODE');
    $payment_method->set('sips_response_code', $code);
    $payment_method->save();

    // Check if the payment is pending to be processed and.
    if ($payment_response->isSuccessful()) {

      // Update the payment information.
      $payment->setRemoteState('done');
      $payment->set('state', 'capture_completed');
      $payment->save();
      return TRUE;
    }

    // Payment wasn't successful:
    // Update the payment information.
    $payment->setRemoteState('failed');
    $payment->set('state', 'void');
    $payment->save();

    drupal_set_message($this->t('An error occurred in the SIPS platform: [@code] @error',
      [
        '@error' => $this->getResponseCodeDescription($code),
        '@code' => $code,
      ]), 'error');

    throw new PaymentGatewayException();
  }

  /**
   * Get the SIPS response description.
   *
   * @param string $code
   *   Response code.
   *
   * @return string
   *   Description for the response code.
   */
  protected function getResponseCodeDescription($code) {
    $descriptions = [
      '00' => 'Authorisation accepted',
      '02' => 'Authorisation request to be performed via telephone with the issuer, as the card authorisation threshold has been exceeded, if the forcing is authorised for the merchant',
      '03' => 'Invalid distance selling contract',
      '05' => 'Authorisation refused',
      '12' => 'Invalid transaction, verify the parameters transferred in the request.',
      '14' => 'invalid bank details or card security code',
      '17' => 'Buyer cancellation',
      '24' => 'Operation impossible. The operation the merchant wishes to perform is not compatible with the status of the transaction.',
      '25' => 'Transaction not found in the Sips database',
      '30' => 'Format error',
      '34' => 'Suspicion of fraud',
      '40' => 'Function not supported: the operation that the merchant would like to perform is not part of the list of operations for which the merchant is authorised',
      '51' => 'mount too high',
      '54' => 'Card is past expiry date',
      '60' => 'Transaction pending',
      '63' => 'Security rules not observed, transaction stopped',
      '75' => 'Number of attempts at entering the card number exceeded',
      '90' => 'Service temporarily unavailable',
      '94' => 'Duplicated transaction: for a given day, the TransactionReference has already been used',
      '97' => 'Timeframe exceeded, transaction refused',
      '99' => 'Temporary problem at the Sips Office Server level',
    ];

    if (empty($descriptions[$code])) {
      return "Unknown error code - [{$code}]";
    }

    return $descriptions[$code];
  }

}
